from http import HTTPStatus
from typing import Any

import httpx

from ... import errors
from ...client import Client
from ...models.image import Image
from ...types import Response


def _get_kwargs(
    resource_type: str,
    image_name: str,
) -> dict[str, Any]:
    _kwargs: dict[str, Any] = {
        "method": "get",
        "url": f"/images/{resource_type}/{image_name}",
    }

    return _kwargs


def _parse_response(*, client: Client, response: httpx.Response) -> Image | None:
    if response.status_code == 200:
        response_200 = Image.from_dict(response.json())

        return response_200
    if client.raise_on_unexpected_status:
        raise errors.UnexpectedStatus(response.status_code, response.content)
    else:
        return None


def _build_response(*, client: Client, response: httpx.Response) -> Response[Image]:
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(client=client, response=response),
    )


def sync_detailed(
    resource_type: str,
    image_name: str,
    *,
    client: Client,
) -> Response[Image]:
    """Get container image

     Returns detailed information about a container image including all available tags, creation dates,
    and size information.

    Args:
        resource_type (str):
        image_name (str):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Image]
    """

    kwargs = _get_kwargs(
        resource_type=resource_type,
        image_name=image_name,
    )

    response = client.get_httpx_client().request(
        **kwargs,
    )

    return _build_response(client=client, response=response)


def sync(
    resource_type: str,
    image_name: str,
    *,
    client: Client,
) -> Image | None:
    """Get container image

     Returns detailed information about a container image including all available tags, creation dates,
    and size information.

    Args:
        resource_type (str):
        image_name (str):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Image
    """

    return sync_detailed(
        resource_type=resource_type,
        image_name=image_name,
        client=client,
    ).parsed


async def asyncio_detailed(
    resource_type: str,
    image_name: str,
    *,
    client: Client,
) -> Response[Image]:
    """Get container image

     Returns detailed information about a container image including all available tags, creation dates,
    and size information.

    Args:
        resource_type (str):
        image_name (str):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Image]
    """

    kwargs = _get_kwargs(
        resource_type=resource_type,
        image_name=image_name,
    )

    response = await client.get_async_httpx_client().request(**kwargs)

    return _build_response(client=client, response=response)


async def asyncio(
    resource_type: str,
    image_name: str,
    *,
    client: Client,
) -> Image | None:
    """Get container image

     Returns detailed information about a container image including all available tags, creation dates,
    and size information.

    Args:
        resource_type (str):
        image_name (str):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Image
    """

    return (
        await asyncio_detailed(
            resource_type=resource_type,
            image_name=image_name,
            client=client,
        )
    ).parsed
